.n64
.create "\RelN64\cart.N64",0x80000000

	.org 0x80000000 	;Cartridge Start

	.db 0x80,0x37,0x12,0x40 	;0x0000 - PI_BSB x4
	.dw 0x0000000F 				;0x0004 - Clock Rate
	.dw ProgramStart 			;0x0008 - Program Start address
	.dw 0x00001444 				;0x000C - Release Offset
	.dw 0,0						;0x0010 - CRC1+2
	.dw 0,0 					;0x0018 - Unknown
	;    12345678901234567890
	.db "WWW.LEARNASM.NET    "	;0x0020 - Cartridge Title

	.dw 0						;0x0034 - Unknown
	.dw 0	 					;0x0038 - Manufacturer ID
	.db 0,0						;0x003C - Cartridge ID
	.db 'J',0					;0x003E - Country Code (E=Usa / J=Japan / P=Europe)

;0x80000040-0x80001000 Bootstrap (4032 bytes)
	.incbin "\ResN64\Boot.bin"		;Generic header ripped from actual game

ProgramStart:
 
.definelabel UserRam,0xA0010000
  
	li a0,0xA4400000
  
	li t0,2					;16 bit
	sw t0,0(a0)			;0xA4400000	VI_STATUS_REG	VI status/control (Read/Write)
  
	li t0,0xA0100000 		;VRAM Base
	sw t0,4(a0)			;0xA4400004	VI_ORIGIN_REG	VI origin (Read/Write)
  
	li t0,320   			;Width
	sw t0,8(a0)			;0xA4400008	VI_WIDTH_REG	VI width (Read/Write)
  
	li t0,0x200    			;Vertical Interrupt (Half line)
	sw t0,12(a0)		;0xA440000C	VI_INTR_REG	VI vertical intr (Read/Write)
  
	li t0,0        			;Current half line
	sw t0,16(a0)		;0xA4400010	VI_CURRENT_REG	VI current vertical line (Read/Write)

	li t0,0x03E52239   		;0xCVBW		C=Color burst start V=Vert sync width(halflines) 
										;B=ColorBurst in pixels W=Hync Width in pixels
	sw t0,20(a0)		;0xA4400014	VI_BURST_REG	VI video timing (Read/Write)
  
	li t0,0x20D    			;Halflines per field
	sw t0,24(a0)		;0xA4400018	VI_V_SYNC_REG	VI vertical sync (Read/Write)
  
	li t0,0x00000C15  		;0x--LLTTT	T=Total linr duration in .25 pixel / L=leap pattern
	sw t0,28(a0)		;0xA440001C	VI_H_SYNC_REG	VI horizontal sync (Read/Write)
  
	li t0,0x0C150C15  		;0x-HHH-hhh	h+H are identical to hsync period
	sw t0,32(a0)		;0xA4400020	VI_LEAP_REG	VI horizontal sync leap (Read/Write)
  
	li t0,0x006C02EC    	;0x-SSS-EEE S=Start of Horiz active screen / E=end of screen
	sw t0,36(a0)		;0xA4400024	VI_H_START_REG	VI horizontal video (Read/Write)

	li t0,0x002501FF     	;0x-SSS-EEE S=Start of Vert active screen / E=end of screen
	sw t0,40(a0)		;0xA4400028	VI_V_START_REG	VI vertical video (Read/Write)
  
	li t0,0x000E0204 		;0x-SSS-EEE S=Start of Vert color burst / E=end of burst
	sw t0,44(a0)		;0xA440002C	VI_V_BURST_REG	VI vertical burst (Read/Write)
  
	li t0,0x00000200		;0xOOOOSSSS O=Horiz subpixel offset / =Scaleup factor
	sw t0,48(a0)		;0xA4400030	VI_X_SCALE_REG	VI x-scale (Read/Write)
  
	li t0,0x00000400		;0xOOOOSSSS O=Vert subpixel offset / =Scaleup factor
	sw t0,52(a0)		;0xA4400034	VI_Y_SCALE_REG	VI y-scale (Read/Write)
  

.include "\SrcAll\BasicMacros.asm"
  
CursorX equ 0x100 		;Defined symbols CursorX EQUals 0x100
CursorY equ 0x101

	jal Cls				;Zero Graphics cursor pos
	nop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
 ; 0x0480 0000 to 0x048F FFFF  Serial interface (SI) registers:
 ; ------------------------------------------------------------
        ; SI_BASE_REG - 0x04800000

        ; 0x0480 0000 to 0x0480 0003  SI_DRAM_ADDR_REG
                 ; SI DRAM address
          ; (R/W): [23:0] starting RDRAM address
        ; 0x0480 0004 to 0x0480 0007  SI_PIF_ADDR_RD64B_REG
                 ; SI address read 64B
            ; (W): [] any write causes a 64B DMA write
        ; 0x0480 0008 to 0x0480 000B  Reserved
        ; 0x0480 000C to 0x0480 000F  Reserved
        ; 0x0480 0010 to 0x0480 0013  SI_PIF_ADDR_WR64B_REG
                 ; SI address write 64B
            ; (W): [] any write causes a 64B DMA read
        ; 0x0480 0014 to 0x0480 0017  Reserved
        ; 0x0480 0018 to 0x0480 001B  SI_STATUS_REG
                 ; SI status
            ; (W): [] any write clears interrupt
            ; (R): [0] DMA busy
                 ; [1] IO read busy
                 ; [2] reserved
                 ; [3] DMA error
                 ; [12] interrupt
        ; 0x0480 001C to 0x048F FFFF  Unused
  
	jal NewLine         ;Move down a line
	nop
	
    
	li t7,UserRam		;Address
	li t6,2				;Line Count
	
;SI=Serial interface
  
;Init Pif 
    li a0,0xBFC007C0		;A0 = PIF Base Register ($BFC00000)
	li t0,8					;Init the PIF
	sw t0,0x3C(a0)			;BFC007FC: Status Register
  
  
  
;Init Joypad for read
	li a0,0xA4800000 	;Serial interface (SI) registers
	li t0,0x1FFFFFFF	;Physical Addr Mask
	
	li a1,PIF_Init		;Init sequence
	and a1,a1,t0		;Convert Virtual Addr-> physical
	sw a1,0x0000(a0) 	;SI_DRAM_ADDR_REG
	
	li a1,0xBFC007C0 	;PIF_RAM_START
	and a1,a1,t0		;Convert Virtual Addr-> physical
	sw a1,0x0010(a0) 	;SI_PIF_ADDR_WR64B_REG + Start WRITE (64 Bytes)
   
  

InfLoop:

;Poll joypad
	li a0,0xA4800000 		;Serial interface (SI) registers
	li t0,0x1FFFFFFF		;Physical Addr Mask
	
	li a1,PIF_JoyState 		;RAM for Joy status write
	and a1,a1,t0			;Convert Virtual Addr-> physical
	sw a1,0x0000(a0) 		;SI_DRAM_ADDR_REG
	
	li a1,0xBFC007C0 		;PIF_RAM_START
	and a1,a1,t0			;Convert Virtual Addr-> physical
	sw a1,0x0004(a0) 		;SI_PIF_ADDR_RD64B_REG + Start READ (64 Bytes)
	

; PIF (JoyChannel) RAM:
; ---------------------
	; 0x1FC0 07C0  PIF_RAM_START
	; 0x1FC0 07C4 (16 low bits)   Status of controller:
		; %X  X  X  X  X  X  X  X  X  X  X  X  X  X  X  X
		 ; A  B  Z  ST U  D  L  R  ?  ?  PL PR CU CD CL CR
		 ; A,B,Z,ST    = A,B,Z, Start buttons
		 ; U,D,L,R     = Joypad directions
		 ; ?,?         = Unknown
		 ; PL,PR       = Pan left, Pan right buttons
		 ; CU,CD,CL,CR = C buttons (up,down,left,right)
	; 0x1FC0 07FF  PIF_RAM_END

	jal cls
	nop
		
	li t7,PIF_JoyState	;Address
	li t6,8				;Line Count
	
	jal MemDump			;Dump Ram to screen
	nop
	
	
	j InfLoop			;Wait forever
	nop 				;Delay Slot
  
  
  .align 8 
	
PIF_Init:
  .dw 0xFF010401,0		;Controller 1
  .dw 0xFF010401,0		;Controller 2
  .dw 0xFF010401,0		;Controller 3
  .dw 0xFF010401,0		;Controller 4
  
  .dw 0xFE000000,0		;Controller End
  .dw 0,0
  .dw 0,0
  .dw 0,1				;Last word is Status reg
  
  
PIF_JoyState: 			;We'll use this to read back joy state
  .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
  
  
 ; additional controller info:

; init controller:
; rdram -> pif
; ff010300
; ffffffff ... for each controller
; fe000000 ... at the end (after controller 4)

; than pif ram is set to:
; ????EE??
; TTTTSS?? ... for each controller

; ?? ... ? zero (0x00)
; EE ... error field
       ; 0x80: no response (no controller connected)
       ; 0x40: Overrun error (?)
       ; 0x00: no error (controller connected)
; TTTT
   ; ... type field
       ; 0x0001 absolute
       ; 0x0002 relative
       ; 0x0004 joyport
       ; 0x8000 eeprom !!!
; SS ... status field
       ; 0x01 card on
       ; 0x02 card pull
       ; 0x04 addr crc err
       ; 0x80 eeprom busy


; get controller data:
; rdram -> pif
; ff010400
; ffffffff ... for each controller
; fe000000 ... at the end (after controller 4)

; than pif ram is set to (!!!attention!!! pif ram is actualized every read):
; 0xbfc007c0: ????EE??
; 0xbfc007c4: BBBBXXYY
; 0xbfc007c8: |
; 0xbfc007cc: |_same as above but for controller 2
    ; ...
; BBBB ... buttons (you have that in controll.txt)
; XX   ... stick x pos [-128..127] or 0x80 (left) .. 0x00 (middle) .. 0x7f (right)
; YY   ... stick y pos [-128..127] or 0x80 (down) .. 0x00 (middle) .. 0x7f (up)
; !!!attention!!!
; all pur versions seem NOT to have implemented the SIGNED XX and YY bytes
; correct.
; 0x80 0x81 0x82 ... 0xff 0x00 0x01 .. 0x7c 0x7e 0x7f
; -128 -127 -126       -1    0    1     125  126  127
   ; left pos           middle pos        right pos
   ; down pos           middle pos          up pos
; !!!attention!!!
  
DoPause:	
	addi t2,t2,-1
	bne t2,zero,DoPause
	nop
	jr ra
	nop
	
JoyWait:	
	lw t2,4(t1)					;Status
	nop
	andi t2,t2,0b00000010		;
	beq t2,zero,JoyWait
	nop
	jr ra
	
MonitorJoy:
	;lb a0,0(t1)
	lbu a0,0(t1)			;Data
	
	lw a1,4(t1)  		;Status
MonitorA0A1A2:
	push ra
	pushAs
	pushTs
	    li t5,'a'
		li t6,'0'
		move t7,a0
		jal ShowRegTextB                ;A5 A6 : A7
		nop
		li t5,'a'
		li t6,'1'
		move t7,a1
		jal ShowRegTextB                ;A5 A6 : A7
		nop
		li t5,'a'
		li t6,'2'
		move t7,a2
		jal ShowRegTextB                ;A5 A6 : A7
		nop
		jal NewLine         
		nop
	popTs
	popAs
	pop ra
	jr ra
	nop	
	
	  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
MyFont:		
	.incbin "\ResN64\ChibiAkumas.fnt"	;Included binary data


Cls:
	li t7,UserRam
  
	li t8,0
	sb t8,CursorX(t7)
	sb t8,CursorY(t7)
	jr ra
	nop

	
	
PrintChar:	;PrintChar A1
    pushAll
	mflo t0
	push t0
	mfhi t0
	push t0
	
		subiu a1,32		;No Characters below 32
	
		li t7,UserRam
		
		la t5,0xA0100000+320*2*8*2	;Screen Base+2 lines
		  
		lbu t8,CursorX(t7)
		li t3,16			;Bytes per Char
		multu t3,t8		;t8=XPos
		mflo t3
		add t5,t5,t3
		addiu t8,1
		sb t8,CursorX(t7)	;Move across 1
		  
		lbu t8,CursorY(t7)
		li t3,320*2*8		;Bytes per line+Char
		multu t3,t8		;A1=Ypos
		mflo t3
		add t5,t5,t3
		  
		la t6,MyFont 	;Font Address
    
		li t2,8			;Height in lines
	
		sll a1,7 		;8*8*2 (2 bytes per pixel) 
		addu t6,a1 		;add to font base

PrintChar_NextLine:	
		li t1,8			;Width in pixels
PrintChar_NextByte:
		lh t4,0(t6) 	;Get 16 bits from font
		addiu t6,2
		sh t4,0(t5) 	;Store 16 bits to screen
		addiu t5,2
	
		subiu t1,1 		;Repeat for next char
		bnez t1,PrintChar_NextByte
		nop
	  
		addiu t5,(320-8)*2 ;Move down 1 line
      
		subiu t2,1 		;Repeat for next line
		bnez t2,PrintChar_NextLine
		nop
	pop t0
	mthi t0
	pop t0
	mtlo t0
	popAll
	jr ra
	nop

	
	
	
PrintString:				;Print String A0
	pushAll		
DrawChars:		
		lbu a1,0(a0) 	;Load a character
		addiu a0,1 		;Move to next Char
		
		li t0,255		;Compare to 255
		beq t0,a1,PrintString_Done	;Done?
		nop
		
		jal PrintChar	;Show Character A0 to the screen
		nop
		
		j DrawChars		;Repeat
		nop
PrintString_Done:	
	popAll				
	jr ra				;Return
	nop
	
NewLine:
	Push t8
	Push t7
		li t7,UserRam
	
		lbu t8,CursorY(t7)
		nop						;Load delay Slot
		addiu t8,1
		sb t8,CursorY(t7)		;Move Down
	
		li t8,0
		sb t8,CursorX(t7)		;Zero Xpos
	pop t7
	pop t8
	jr ra
	nop
	

.include "\SrcAll\monitor.asm"  	


.org 0x80000000+1052672-1			;Pad to 8 megabit
	.db 0
.close

