

.n64							;Tell The assembler what system 
									;Were' building for
									
.create "\RelN64\cart.N64",0x80000000	;File and Base address

	.org 0x80000000 	;Cartridge Start

	
	.db 0x80,0x37,0x12,0x40 	;0x0000 - PI_BSB x4
	.dw 0x0000000F 				;0x0004 - Clock Rate
	.dw ProgramStart 			;0x0008 - Program Start address
	.dw 0x00001444 				;0x000C - Release Offset
	.dw 0,0						;0x0010 - CRC1+2
	.dw 0,0 					;0x0018 - Unknown
	;    12345678901234567890
	.db "WWW.LEARNASM.NET    "	;0x0020 - Cartridge Title

	.dw 0						;0x0034 - Unknown
	.dw 0	 					;0x0038 - Manufacturer ID
	.db 0,0						;0x003C - Cartridge ID
	.db 'J',0					;0x003E - Country Code 
									;(E=Usa / J=Jpn / P=Eur)
	
	
;0x80000040-0x80001000 Bootstrap (4032 bytes)
	.incbin "\ResN64\Boot.bin"		;Generic header ripped from
										;an actual game

	
ProgramStart:
 
;Let's set up our screen

	li a0,0xA4400000	;VI_BASE_REG - Video Interface Registers
  
	li t0,2				;16 bit
	sw t0,0(a0)			;0xA4400000	VI_STATUS_REG	VI status/control (Read/Write)
  
	li t0,0xA0100000 	;VRAM Base
	sw t0,4(a0)			;0xA4400004	VI_ORIGIN_REG	VI origin (Read/Write)
  
	li t0,320   		;Width
	sw t0,8(a0)			;0xA4400008	VI_WIDTH_REG	VI width (Read/Write)
  
	li t0,0x200    		;Vertical Interrupt (Half line)
	sw t0,12(a0)		;0xA440000C	VI_INTR_REG	VI vertical intr (Read/Write)
  
	li t0,0        		;Current half line
	sw t0,16(a0)		;0xA4400010	VI_CURRENT_REG	VI current vertical line (Read/Write)

	li t0,0x03E52239   	;0xCVBW		C=Color burst start V=Vert sync width(halflines) 
									;B=ColorBurst in pixels W=Hync Width in pixels
	sw t0,20(a0)		;0xA4400014	VI_BURST_REG	VI video timing (Read/Write)
  
	li t0,0x20D    		;Halflines per field
	sw t0,24(a0)		;0xA4400018	VI_V_SYNC_REG	VI vertical sync (Read/Write)
  
	li t0,0x00000C15  	;0x--LLTTT	T=Total linr duration in .25 pixel / L=leap pattern
	sw t0,28(a0)		;0xA440001C	VI_H_SYNC_REG	VI horizontal sync (Read/Write)
  
	li t0,0x0C150C15  	;0x-HHH-hhh	h+H are identical to hsync period
	sw t0,32(a0)		;0xA4400020	VI_LEAP_REG	VI horizontal sync leap (Read/Write)
  
	li t0,0x006C02EC    ;0x-SSS-EEE S=Start of Horiz active screen / E=end of screen
	sw t0,36(a0)		;0xA4400024	VI_H_START_REG	VI horizontal video (Read/Write)

	li t0,0x002501FF    	;0x-SSS-EEE S=Start of Vert active screen / E=end of screen
	sw t0,40(a0)		;0xA4400028	VI_V_START_REG	VI vertical video (Read/Write)
  
	li t0,0x000E0204 	;0x-SSS-EEE S=Start of Vert color burst / E=end of burst
	sw t0,44(a0)		;0xA440002C	VI_V_BURST_REG	VI vertical burst (Read/Write)
  
	li t0,0x00000200	;0xOOOOSSSS O=Horiz subpixel offset / =Scaleup factor
	sw t0,48(a0)		;0xA4400030	VI_X_SCALE_REG	VI x-scale (Read/Write)
  
	li t0,0x00000400	;0xOOOOSSSS O=Vert subpixel offset / =Scaleup factor
	sw t0,52(a0)		;0xA4400034	VI_Y_SCALE_REG	VI y-scale (Read/Write)
  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;       RRRRRGGGGGBBBBB-
	li a0,0b0000000000000000	;Black
	
	jal Cls				;Clear the screen color A0
	nop

	li a0,16		;Xpos
	li a1,8			;Ypos
	
	jal GetScreenPos	;A0= Vram Dest for XY (A0,A1)
	nop
  	
	la a1,Bitmap		;Source Bitmap 
		;Format: (%RRRRRGGGGGBBBBBB- BigEndian)
		
	li a2,48			;Width
	li a3,48			;Height
	
	jal DrawBitmap		;Draw bitmap to screen address A0
	nop
	

InfLoop:
	j InfLoop			;Wait forever
	nop 				;Delay Slot
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Cls:
	la t0,0xA0100000	;Screen Base
	li t1,320*240		;Total Screen pixels
	
ClsAgain:	
	sh a0,0(t0) 		;Store 16 bits to screen (one pixel)
	addiu t0,2			;Move to next pixel
	
	addi t1,t1,-1
	bnez t1,ClsAgain	;Decrease Loopcount and repeat
	nop
	
	jr ra
	nop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;Screen is 320x240 @ 16bpp 
;Screen base is 0xA0100000
;BUT first 14 lines are offscreen!
  
GetScreenPos:		;A0=Xpos A1=Ypos
					;Return A0 = Vram pos

	sll t0,a0,1			;Xpos *2
	
	la a0,0xA0100000+320*2*14	;Top of screen
	add a0,t0			;Add Xpos in bytes
	
	li t0,320*2			;Bytes per line (640 bytes)
	multu t0,a1			;Ypos * Line Width
	
	mflo t0
	add a0,t0			;Add Ypos in bytes

	jr ra
	nop
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

				  ;%FEDCBA9876543210
;Bitmap format is %RRRRRGGGGGBBBBBB- BigEndian

Bitmap:	;Our 'Chibiko' mascot... Yay!
		.incbin "\resN64\RawRGB_bigendian.RAW" 
		
	.align 4	;Ensure we're 32 bit aligned or 
					;our code won't work


	
DrawBitmap:	;A0=VRAM Dest   A1=Bitmap   A2=Width   A3=Height

DrawBitmapY:	
	move t1,a0		;VRAM Dest
	move t2,a2		;Width
DrawBitmapX:	
	lh t0,(a1)				;Load a pixel
	sh t0,(t1)				;Draw a pixel

	addiu a1,2				;Next Source Pixel
	addiu t1,2				;Across one screen pixel
	
	subiu t2,1				;Width Loop
	bnez t2,DrawBitmapX
	nop
  
	addi a0,320*2	;Down one line (320 pixels)
  
	subiu a3,1		;Height Loop
	bnez a3,DrawBitmapY
	nop
		
	jr ra
	nop
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
					

.org 0x80000000+1052672-1			;Pad to 8 megabit
	.db 0
.close

