.include "\SrcAll\BasicMacros.asm"

.n64							;Tell The assembler what system 
									;Were' building for
									
.create "\RelN64\cart.N64",0x80000000	;File and Base address

	.org 0x80000000 	;Cartridge Start

	
	.db 0x80,0x37,0x12,0x40 	;0x0000 - PI_BSB x4
	.dw 0x0000000F 				;0x0004 - Clock Rate
	.dw ProgramStart 			;0x0008 - Program Start address
	.dw 0x00001444 				;0x000C - Release Offset
	.dw 0,0						;0x0010 - CRC1+2
	.dw 0,0 					;0x0018 - Unknown
	;    12345678901234567890
	.db "WWW.LEARNASM.NET    "	;0x0020 - Cartridge Title

	.dw 0						;0x0034 - Unknown
	.dw 0	 					;0x0038 - Manufacturer ID
	.db 0,0						;0x003C - Cartridge ID
	.db 'J',0					;0x003E - Country Code 
									;(E=Usa / J=Jpn / P=Eur)
	
	
;0x80000040-0x80001000 Bootstrap (4032 bytes)
	.incbin "\ResN64\Boot.bin"		;Generic header ripped from
										;an actual game

	
ProgramStart:
 
;Let's set up our screen

	li a0,0xA4400000	;VI_BASE_REG - Video Interface Registers
  
	li t0,2				;16 bit
	sw t0,0(a0)			;0xA4400000	VI_STATUS_REG	VI status/control (Read/Write)
  
	li t0,0xA0100000 	;VRAM Base
	sw t0,4(a0)			;0xA4400004	VI_ORIGIN_REG	VI origin (Read/Write)
  
	li t0,320   		;Width
	sw t0,8(a0)			;0xA4400008	VI_WIDTH_REG	VI width (Read/Write)
  
	li t0,0x200    		;Vertical Interrupt (Half line)
	sw t0,12(a0)		;0xA440000C	VI_INTR_REG	VI vertical intr (Read/Write)
  
	li t0,0        		;Current half line
	sw t0,16(a0)		;0xA4400010	VI_CURRENT_REG	VI current vertical line (Read/Write)

	li t0,0x03E52239   	;0xCVBW		C=Color burst start V=Vert sync width(halflines) 
									;B=ColorBurst in pixels W=Hync Width in pixels
	sw t0,20(a0)		;0xA4400014	VI_BURST_REG	VI video timing (Read/Write)
  
	li t0,0x20D    		;Halflines per field
	sw t0,24(a0)		;0xA4400018	VI_V_SYNC_REG	VI vertical sync (Read/Write)
  
	li t0,0x00000C15  	;0x--LLTTT	T=Total linr duration in .25 pixel / L=leap pattern
	sw t0,28(a0)		;0xA440001C	VI_H_SYNC_REG	VI horizontal sync (Read/Write)
  
	li t0,0x0C150C15  	;0x-HHH-hhh	h+H are identical to hsync period
	sw t0,32(a0)		;0xA4400020	VI_LEAP_REG	VI horizontal sync leap (Read/Write)
  
	li t0,0x006C02EC    ;0x-SSS-EEE S=Start of Horiz active screen / E=end of screen
	sw t0,36(a0)		;0xA4400024	VI_H_START_REG	VI horizontal video (Read/Write)

	li t0,0x002501FF    	;0x-SSS-EEE S=Start of Vert active screen / E=end of screen
	sw t0,40(a0)		;0xA4400028	VI_V_START_REG	VI vertical video (Read/Write)
  
	li t0,0x000E0204 	;0x-SSS-EEE S=Start of Vert color burst / E=end of burst
	sw t0,44(a0)		;0xA440002C	VI_V_BURST_REG	VI vertical burst (Read/Write)
  
	li t0,0x00000200	;0xOOOOSSSS O=Horiz subpixel offset / =Scaleup factor
	sw t0,48(a0)		;0xA4400030	VI_X_SCALE_REG	VI x-scale (Read/Write)
  
	li t0,0x00000400	;0xOOOOSSSS O=Vert subpixel offset / =Scaleup factor
	sw t0,52(a0)		;0xA4400034	VI_Y_SCALE_REG	VI y-scale (Read/Write)
  
  
.definelabel UserRam,0xA0010000 	;Address of our program ram

CursorX equ 0x100 		;Defined symbols CursorX EQUals 0x100
CursorY equ 0x101

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	jal Cls				;Zero Graphics cursor pos
	nop

	la a0,text 			;A0 = String (255 terminated)
	jal PrintString		;Print String 
	nop
  
	jal NewLine         ;Move down a line
	nop
	
	jal Monitor			;Show Registers to screen
	nop
	
    
	li t7,UserRam		;Address
	li t6,2				;Line Count
	
	jal MemDump			;Dump Ram to screen
	nop
  
InfLoop:
	j InfLoop			;Wait forever
	nop 				;Delay Slot
  
Text:
	.db "Hello World!",255
	.align 4 			;Align 32-Bit

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
Cls:
	la t5,0xA0100000;+320*16*2	;Screen Base + 16 lines

	    ;   RRRRRGGGGGBBBBB-
	li t4,0b0000000000011110	;DarkBlue
	
	li t3,320*240		;Screen pixels
	
ClsAgain:	
	sh t4,0(t5) 		;Store 16 bits to screen (one pixel)
	addiu t5,2			;Move to next pixel
	
	addi t3,t3,-1
	bnez t3,ClsAgain	;Decrease Loopcount and repeat
	
		
	li t7,UserRam		;Point T7 to our ram
  
	li t8,0
	sb t8,CursorX(t7)	;Set Xpos=0
	sb t8,CursorY(t7)	;Set Ypos=0
	jr ra
	nop


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
MyFont:		
	.incbin "\ResALL\Font96.fnt"
	
PrintChar:	;PrintChar A1
	pushall
		mflo t0
		push t0
		mfhi t0
		push t0
			subiu a1,32		;No Characters below 32
			
			la t6,MyFont 	;Font Address
			
			sll a1,3 		;*3 (8 bytes per tile) 
			addu t6,a1 		;add to font base
			
			
			li t7,UserRam
			
			la t5,0xA0100000+320*16*2	;Screen Base+2 char lines
			  
			lbu t8,CursorX(t7)
			li t3,16			;Bytes per Char
			multu t3,t8			;Xpos * 16
			mflo t3				
			add t5,t5,t3		;Update Vram Pos
			
			addiu t8,1
			sb t8,CursorX(t7)	;Move across 1
			  
			lbu t8,CursorY(t7)
			li t3,320*8*2		;Bytes per line+Char
			multu t3,t8			;Ypos*320*8*2
			mflo t3
			add t5,t5,t3		;Update Vram Pos
			
			
			li t2,8			;Height in lines
		PrintChar_NextLine:	
			li t1,8			;Width in pixels
			
			lb t3,0(t6)		;Get a Font Line
			addiu t6,1		;Move to next font line
			
		PrintChar_NextByte:
				;   RRRRRGGGGGBBBBB-
			li t4,0b0000000000011110	;OFF pixel (Blue)
			
			andi t0,t3,0b10000000	;Test bit of Font line
			beqz t0,PrintChar_DrawPixel	;Pixel=0?
			nop
			
			li t4,0b1111111111000000	;ON pixel (Yellow)
			
		PrintChar_DrawPixel:		
			sh t4,0(t5) 	;Store 16 bits to screen
			addiu t5,2
			
			sll t3,1		;Move to next bit of Font line

			subiu t1,1 		;Repeat for next char
			bnez t1,PrintChar_NextByte
			nop
			
		  
			addiu t5,(320-8)*2 ;Move down 1 line
		  
			subiu t2,1 		;Repeat for next line
			bnez t2,PrintChar_NextLine
		pop t0
		mthi t0
		pop t0
		mtlo t0
	popall
	jr ra
	nop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  	
PrintString:				;Print String A0
	push ra
	push a0
	push a1
	push t0
DrawChars:		
		lbu a1,0(a0) 	;Load a character
		addiu a0,1 		;Move to next Char
		
		li t0,255		;Compare to 255
		beq t0,a1,PrintString_Done	;Done?
		nop
		
		jal PrintChar	;Show Character A0 to the screen
		nop
		
		j DrawChars		;Repeat
		nop
PrintString_Done:	
	pop t0
	pop a1
	pop a0
	pop ra			
	jr ra				;Return
	nop
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  	
NewLine:
	Push t8
	Push t7
		li t7,UserRam			;Get address of ram vars

		lbu t8,CursorY(t7)
		nop						;Load delay Slot

		addiu t8,1				;Move Down
		sb t8,CursorY(t7)		

		li t8,0
		sb t8,CursorX(t7)		;Zero Xpos
	pop t7
	pop t8
	jr ra
	nop
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  	

.include "\SrcAll\monitor.asm"  	


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  	
.org 0x80000000+1052672-1			;Pad to 8 megabit
	.db 0
.close

