	;.ifndef vm_remap_TableAddr

	;.endif

VM_StackTop equ 0x200
	
ChibiVM_Init:
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack

;Clear Low Ram
		la a0,vm_rambaseAddr	;Address
		lw a0,(a0)
		nop
		
		move s2,a0
		li s1,512/4
		
ChibiVM_Init_Clear:	
		sw zero,(s2)			;Clear first 512 bytes of VM Ram
		addi s2,4
		addi s1,s1,-1
		bne s1,zero,ChibiVM_Init_Clear
		nop
		
		
		
;Set defaults
		.ifdef vm_traps
			la s0,vm_traps
			la a1,vm_trap_TableAddr
			sw s0,(a1)
			
			lbu s1,(s0)	 ;R6=Execute address from fixed trap table
			addi s0,s0,1
			lbu s2,(s0)
			addi s0,s0,1
			sll s2,s2,8
			
			or s6,s1,s2
			add s6,s6,a0
		.else 
			li s6,VM_ProgLoadAddr	;R6 = Execute address
			add s6,s6,a0
		.endif
		
		li s7,VM_StackTop			;R7 = Stack Pointer
		add s7,s7,a0
		
		li s5,VM_ProgLoadAddr		;R5 = Load Address
		add s5,s5,a0				;     (Rom copy address)
		

;Default Memory Mapping
		li t0,0x0000
		StoreLEA0 t0,vm_rBank0
		li t0,0x4000
		StoreLEA0 t0,vm_rBank1
		li t0,0x8000
		StoreLEA0 t0,vm_rBank2
		li t0,0xC000
		StoreLEA0 t0,vm_rBank3
		
		
;Seek a ROM header 
		la t8,VM_Program	;Start of program ROM
		jal FindHeader 				;Look for a header
		nop
				
;Update VM state from header (if found, else defaults)
		la a0,vm_rambaseAddr		;Address
		lw a0,(a0)
		
		StoreLEA0 s7,VM_rSP			;Stack Pointer (R7)
		StoreLEA0 s6,VM_rPC			;Program counter (R6)

;Transfer the program to VM RAM
		la s3,VM_Program
		li s1,VM_Program_End-VM_Program
		
		
		
ProgramCopyAgain:	
		lbu s0,(s3)		;Get a byte
		addi s3,s3,1
		sb s0,(s5)		;Save a byte
		addi s5,s5,1

		addi s1,s1,-1
		bne s1,zero,ProgramCopyAgain
		nop

	lw ra,0(sp)		;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
HeaderMagicKey: .db "!!!!Chibi/VM!!!!"
				
	
FindHeader:			;R9=Search Start  R6=PC R7=SP  R5=Load
	move s3,t8
	li s5,256
MagicKeySearchAgain:	
	la a1,HeaderMagicKey		;Key to find
	move s2,s3
	li s1,16					;bytes to scan
MagicKeySearching:		
	lbu s0,(a1)					;Get a byte (template)
	addi a1,a1,1
	lbu s4,(s2)					;Get a byte (Program)
	addi s2,s2,1

	bne s0,s4,MagicKeyNG		;Check this byte.
	nop
	
	addi s1,s1,-1
	bne s1,zero,MagicKeySearching
	nop
	
	j MagicKeyOK
	nop
	
MagicKeyNG:
	li t0,256
	add s3,s3,t0			;Move to next block
	addi s5,s5,-1			;Try 256 times
	bne s5,zero,MagicKeySearchAgain	;Try again
	nop
	
	
;Mo Magic Key - RETURN to caller
	jr ra					;Return
	nop
		
MagicKeyOK:		;Key Found! Header is at r3

	lw s6,0x40(s3)		;Defalt PC
	nop	
	ConvertEndian s6
	
	move s5,s6			;Load Address
	add s5,s5,a0
	
	lw s7,0x44(s3)		;Defalt SP
	ConvertEndian s7
	
	
	
	lw s2,0x70(s3)		;Trap table offset
	nop
	ConvertEndian s2
	
	beq s2,zero,Header_NoTrapTable
	nop
	
	add s2,s2,t8	;For trap table addr relative to prog start
			
	la a1,vm_trap_TableAddr
	sw s2,(a1)			;Store addr of Trap table in 'ROM'
	
	lbu s6,(s2)			;First entry is execute address
	lbu s0,1(s2)
	nop
	sll s0,8
	add s6,s6,s0		;Get 16 bit address 

	add s6,s6,a0		;Run address from Trap 0
	
	
	
	lw s1,0x48(s3)		;Address table of Traps in RAM
	nop					; (Recommended $0100)
	
	ConvertEndian s1
	
	
	beq s1,zero,Header_NoTrapTable ;0=Don't copy to RAM
	nop
							
	add s1,s1,a0
	
	
		
Header_CopyTrapTable:	;Copy ROM trap table to RAM
	lbu s0,(s2)
	addi s2,s2,1
	sb s0,(s1)
	addi s1,s1,1
	
	lbu s4,(s2)
	addi s2,s2,1
	sb s4,(s1)
	addi s1,s1,1
	
	or t0,s0,s4			; 0x0000 = End of table
	bne t0,zero,Header_CopyTrapTable 
	nop
Header_NoTrapTable:		
	
		
	
;Copy Address space layout
	move s1,s3
	addi s1,s1,0x4C		;Copy Default address map
	
	move s2,a0			;Destination (First bank addr)
	addi s2,s2,vm_rBank0
	li s4,32			;4 * 4 byte addresses
	
Header_CopyAddressMap:	
	lbu s0,(s1)
	addi s1,s1,1
	sb s0,(s2)
	addi s2,s2,1
	addi s4,s4,-1
	bne s4,zero,Header_CopyAddressMap
	nop
;Address Remap table address	

	lw s1,0x74(s3)		;Address Remap Table
	ConvertEndian s1
	
	la a1,Addr_vm_remap_TableAddr
	lw a1,(a1)
	nop
	sw s1,(a1)			;Store Table address in VM settings
		
	;mov pc,lr			;RETURN (All done)
	jr ra				;Return
	nop
	
Addr_vm_remap_TableAddr:	.dw vm_remap_TableAddr