
;This was ported from the ARM which was ported from the Z80! 
	

;s0	r0  A
;s1	r1  B BC 
;s2 r2  D DE
;s3 r3  H HL
;s4	r4  C
;s5	r5  E
;s6 r6  L
;s7 r7  IX  (Was A5 on 68000)

;t7 r8  IY  (Was A6 on 68000)
;t8 r9	 	(Was D7 on 68000)
;t9 r10	 	(Was A0 / r10 on 68000)

;v0=zero flag
;v1=carry flag
	


;ChibiVM simulates an 8 bit cpu, it's design is inspired mostly by the Z80 
;  and 6809	and will be therefore referred to as the AKUC809.R1 (chibi AKUmas Cpu 809 Revision 1)

;It uses the Z80's 8/16 bit register pair combination and core instruction set capabilities, flag functions
; including 16 bit stack pointer and IX+n type addressing - It is also LITTLE ENDIAN
;It uses the 6809's zeropage, three letter instructions mnemnonics, B accumulator and extended byte for flaxible addressing
;It uses the PDP-11's 8 register concept and powerful addressing modes (which also inspired the 6809)
;It uses 68000 style traps for interrupts
;It uses the 2 byte branch ops and vblank counter from Chip-8
;It uses the concept of memory mapped registers of the TMS9900
;It uses Memory mapping / bankswitching based on that of the Enterprise / Mastersystem

	
;Syntax

;First 256 bytes are a 'zeropage' - The zeropage is not relocatable
;First 16 of those bytes are 'register'
;0-7  are register pairs R0-R7
;8-16 are system (SP/PC Flags)
;17-31 are free for user use.
;32-47 are reserved for Virtual memory mapping (when enabled)

;Commands are defined by top 6 bits:
; %CCCCCCAA - C= 6 bit command A=2 bit addressing mode
; So 64 commands max!
; Commands %110000+ are reserved for user extensions.

; AA=0 Params are R0 and R2				 %CCCCCC00
; AA=1 Params are R0 and 8 bit immediate %CCCCCC01 %IIIIIIII
; AA=2 Params are R0 and Zero page Z  	 %CCCCCC10 %ZZZZZZZZ
; AA=3 uses extra Byteparam 			 %CCCCCC11 %PPPPDDDD  - P=Parameter2 D=Param1 & Dest
	;More bytes may be neded depending on parameter selection
	
;Branches use diferent addressing options:
;	AA=0 PC=PC+2
;	AA=1 PC=PC+IMM8
;	AA=2 PC=IMM16
;	AA=3 PC=R6/R7
	
;ByteParam supports 15 options for source & Destination.

;Reg 8 bit 0-3, 16 bit 4,6 (5,7 can be acessed via ZP)
;Zeropage Address / 16 bit address
;RegPair = 16 bit pointer
;RegPair+ = 16 bit pointer with PostInc
;RegPair with imm8 offset = 16 bit pointer with signed immediate offset

;For Parameter only: 	Immediate 8 bit / Immediate 16 bit 
;For Dest only: 		F register / Top of stack

;Warning! Some options may not make sense - Eg 8 bit load command with 16 bit immediate
;16 bit load with 8 bit immediate.
;Results will be unpredictable!

;Flags are in Zeropage:15 in the format ;%HSTEI-ZC
;Flag functions (and all functions in general) should mimic a z80
;Commands that do not set flags leave them unchanged.
;Commands That set one flag may not correctly set both flags, other flag is undefined

;Halt Flag: Set when the CPU halts, can be used to detect when CPU is not running (or check if current command is halt)
;			If you restart the CPU with a new task, you should clear the flags
;			If you HALT the system, you should assume it will never restart (your work is done)
;			If you want to be restarted, set S flag (Sleep) - to tell the host to restart at a good time (By stepping over the HALT command)

;Sleep Flag Use this to mark a HALT as possibly temporary if you want the host to restart your VM at some point

;Trap Flag: you can set this at the start of your traps, and clear it when done
;			This is intended so you can see if the CPU is running an interrupt - and give it higher priority 
;			If doing this, Note the return will not occur when the flag clears, so you should run one extra tick to process the RET command after the flag clears

;			Back up flags before trap with PHF
;			Set Flag with:    vm_ORR+ParamExtByt,Dest_ZeroPg+Param_Imm8,VM_rF,VM_fTrap  
;			Flags should restored with PLF 

;Error Flag 	Set when (1) an unknown command byte was detected
;				(2) A system call or trap was executed with an unsupported parameter (eg bankswitching on a machine that cannot do it)

;Interrupt Disable flag: (I)
;			Stop hardware from sending interrupt traps (Eg Vblank)



;Virtual Memory & Bankswitching:
;			The 64k address range is split into 4 blocks in the format
;			%BBAAAAAA AAAAAAAA BB=Bank AA...AA=Address 
;			4x 32 bit calues in the Zeropage define the Physical address base for each bank
;			This 32 bit value is added to the A bits to give the pysical address
; 			The correct 'normal' values are therefore &0000 &4000 &8000 &C000
;These are HARDWARE addresses, ChipEXT will provide a 'platform independant' bankswitching method 
;WARNING: 	Bankswitching occurs only once for each read/write.
;			This means reading a word spread across a 16k boundary may malfunction, as the second byte will be read from the first bank+1.
;			* Including words in code! *

;Zero page layout:

;First 8 bytes are registers:	AA KK BB CC DD EE HH LL (General Regs)
;Next 8 bytes are system:		FF FU PC PC SP SP VB VB (System Regs)
;Next 16 bytes are for user:    0  0  0  0  0  0  0  0  (User)
;...						    0  0  0  0  0  0  0  0	(User)
;Next 16 bytes are bankswitch   B0 B0 B0 B0 B1 B1 B1 B1 (Bankswitching)
;...						    B2 B2 B2 B2 B3 B3 B3 B3 (Bankswitching)

;Remaining bytes are implementation specific. Stack may share ZP ram. 
;Bankswitch bytes are only used if vm_useVMEM is defined 
; A recomended absolute minimum would be 64 bytes ram total for ZP+Stack

;Traps and systcalls use the 'parameter' to select a numbered function from the vector list
;These are intended to work from an 8 bit immediate, but can also work from a register or memory.

;value in register AK will be used to select 'subfunction' of trap - other regs are parameters as defined by that function

;Traps:
;0 RESET - Used to INIT the program counter
;1 VBLANK?
;3+ - For your own use

;Syscall: (suggestion)
;0 ChipEXT - reserved for system functions (Bankswitching / Debugger / Rand / Mult etc) - comamnd in A, Sets E flag if unknown command
;1 VMControl - For configuring VM functions (Enable/disable interrupts / Shutdown VM / Disk Load etc)
;2 VMMonitor - VM Monitor - dump regs to screen
;3+ - For your own use

;All registers are 8 bit, all can be used as 16 bit pairs
;R0-R3 are intended for 8 bit ops (R2+3 is also ideal for 16 bit)
;R4+5 R6+7 are intended for 16 bit ops
;All are accessible from the Zero page addresses 0-7

;Registers Should be referred to by number, though they are given Z80 style 'Letter pairs' to make them easier to remember
;16 bit ops automatically use the next register (eg R0+R1)
;Acutally, odd combinations will also work in 16 bits (Eg R5+R6 LD), though the indirect addressing modes
;do not support these combos.


;When an operation is performed with no other parameter, it is performed on AB
; EG: XOR   performs A  = A  XOR B (R0 = R0 XOR R2)
;     ADD16 performs AK = AK + UB (R0/1 = R0/1 + R2/3)

		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	.align 4
vm_ParameterAddressingModes:	;Subtract 5 (First addressing R0-R4)
	.dw VM_GetAddress_R6							;vm_pR6 equ 5	;Param R6
	.dw VM_GetAddress_ByteParam_Indir16_R2_BaseImm	;vm_PB2 equ 6 	;Indirect 16 bit pointer R2 (16 bit to anywhere) - Base+imm8 offset
	.dw VM_GetAddress_ByteParam_Indir16_R4_BaseImm	;vm_PB4 equ 7 	;Indirect 16 bit pointer R4 (16 bit to anywhere) - Base+imm8 offset
	.dw VM_GetAddress_ByteParam_Indir16_R2_PostInc	;vm_PU2 equ 8 	;Indirect 16 bit pointer R2 (16 bit to anywhere) - PostInc
	.dw VM_GetAddress_ByteParam_Indir16_R6_PostInc	;vm_PU6 equ 9 	;Indirect 16 bit pointer R6 (16 bit to anywhere) - PostInc
	.dw VM_GetAddress_ByteParam_Indir16_R4			;vm_PQ4 equ 10 	;Indirect 16 bit pointer R4 (16 bit to anywhere)
	.dw VM_GetAddress_ByteParam_Indir16_R6			;vm_PQ6 equ 11 	;Indirect 16 bit pointer R6 (16 bit to anywhere)
	.dw VM_GetAddress_ByteParam_ImmAddr16			;vm_pA6 equ 12	;Address 16
	.dw VM_GetAddress_ByteParam_ImmAddr8			;vm_pA8 equ 13	;Address 8 (ZeroPage)
	.dw VM_GetAddress_ByteParam_Imm16				;vm_pI6 equ 14	;Immediate 16 - Source only
	.dw VM_GetAddress_ByteParam_Imm8				;vm_pI8 equ 15	;Immediate 8 - Source only

vm_PrepVectors:				;Subtract 1 - Zero is nothing
	.dw VM_GetAddress_Branch 		;1 - Branches
	.dw VM_GetAddressPrep			;2 - Basic
	.dw VM_GetAddress_VM_GetFlagC	;3 - Get Carry Flag
	.dw VM_GetAddressPrep16			;4 - Patch for 16 bit imm
	.dw VM_GetAddressPrepZPDest		;5 - Patch for INC ZP
	.dw VM_GetAddressPrepZPDest_GetFlagC ;6 patch for ROLz RORz
	
vm_FinVectors:				;Subtract 1 - Zero is nothing
	.dw VM_PC_Store				;1 store PC
	.dw VM_PCINC				;2 INC PC
	.dw VM_PCINC_FZ				;3 store flags
	.dw VM_PCINC_FZ_LD_IX_E		;4 store flags
	.dw VM_PCINC_FZ_LD_IX_DE	;5 store flags and save 16 bit
	.dw VM_PCINC_LD_IX_E 		;6 save
	.dw VM_PCINC_LD_IX_DE 		;7 save 16 bit

vm_DecoderMatrix:
	.dw cmd_NOP,		0
	.dw cmd_PSH,		fin_VM_PCINC
	.dw cmd_PUL,		fin_VM_PCINC
	.dw cmd_PHF,		fin_VM_PCINC
	.dw cmd_BRA,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.dw cmd_BSR,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.dw cmd_BEQ,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.dw cmd_BNE,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.dw cmd_BCS,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.dw cmd_BCC,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.dw cmd_ADD,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_SUB,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_CMP,		prp_VM_GetAddress+fin_VM_PCINC_FZ
	.dw cmd_MOV,		prp_VM_GetAddress+fin_VM_PCINC_LD_IX_E
	.dw cmd_STO,		prp_VM_GetAddress+fin_VM_PCINC
	.dw cmd_LEA,		prp_VM_GetAddress+fin_VM_PCINC_LD_IX_DE
	.dw cmd_NEG,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_ROL,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_ROR,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_ASL,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_ASR,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_LSR,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_INC,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_DEC,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_AND,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_ORR,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_XOR,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_TRP,		prp_VM_GetAddress+fin_VM_PC_Store
	.dw cmd_SYS,		prp_VM_GetAddress
	.dw cmd_MOV16,		prp_VM_GetAddress16+fin_VM_PCINC_LD_IX_DE
	
	.dw cmd_INC16,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_LD_IX_DE
	.dw cmd_DEC16,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_LD_IX_DE
	
	.dw cmd_ADD16,		prp_VM_GetAddress16+fin_VM_PCINC_FZ_LD_IX_DE
	.dw cmd_SUB16,		prp_VM_GetAddress16+fin_VM_PCINC_FZ_LD_IX_DE
	.dw cmd_SWP16,		prp_VM_GetAddress16+fin_VM_PCINC_LD_IX_DE
	.dw cmd_STO16,		prp_VM_GetAddress16+fin_VM_PCINC
	.dw cmd_BRA16,		prp_VM_GetAddress16+fin_VM_PC_Store	
	.dw cmd_BSR16,		prp_VM_GetAddress16+fin_VM_PC_Store	
	.dw cmd_TST,		prp_VM_GetAddress+fin_VM_PCINC_FZ
	.dw cmd_CMP16,		prp_VM_GetAddress16+fin_VM_PCINC_FZ
	.dw cmd_CLR,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_LD_IX_E
	.dw cmd_CLR,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_LD_IX_DE
	.dw cmd_TOR,		prp_VM_GetAddress+fin_VM_PCINC_FZ
	.dw cmd_ADC,		prp_VM_GetAddress_VM_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.dw cmd_SBC,		prp_VM_GetAddress_VM_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	
VM_Tick:
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the RA register onto the stack

	la a0,vm_rambaseAddr
	lw s3,(a0)			;Get Base of VM Ram
	lw a0,(a0)			;Get Base of VM Ram
	nop
	loadLEA0 vm_rpc,s3		;Load Little endian S3 from A0
							;(Program Counter)
	.ifdef vm_usevmem
		jal vm_virmemgethlbytea
		nop
	.else
		lbu s0,(s3)			;Get A command from the Program
		nop
	.endif
	
	move s2,s0				;D=Command
	andi s0,s0,0b11111100
	
	li t0,vm_lastcomamnd
	bgt s0,t0,cmd_error		;Command not recognized
	nop
	

;Find the command in the decoder matrix	
	la s4,vm_decodermatrix
	sll s0,s0,1				;Command *8
	add s4,s4,s0			
	
;calculate our finisher
	lw s0,4(s4)
	nop
	andi s0,s0,0b00001110	;%-pppfff-
	
	beq s0,zero,vm_nofinish
	nop
	
	la s3,vm_finvectors-4
	sll s0,s0,1			;4 bytes per address
	add t3,s3,s0
	
	lw s0,(t3)			;Get the address of the finisher
	
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw s0,0(sp)			;Run this after the actual command
	
vm_nofinish:	

;Calculate our Comamnd Job
	lw s0,0(s4)			;Get the address of the command itself
	
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw s0,0(sp)			;Run the actual command
	

;Calculate our Prep Job	
	lw s0,4(s4)			;%-pppfff-
	nop	
	andi s0,s0,0b01110000
	
	beq s0,zero,vm_noprep
	nop
	
	la s3,vm_prepvectors-4
	srl s0,s0,2			;4 bytes per address
	add t3,s3,s0
	
	lw s0,(t3)			;Get the Prep routine 
	
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw s0,0(sp)			;run this before the actual command
	
	
vm_noprep:

	la a0,vm_rambaseAddr
	lw s3,(a0)			;Get Base of VM Ram
	lw a0,(a0)			;Get Base of VM Ram
	
	loadLEA0 vm_rpc,s3		;Load Little endian S3 from A0
							;(Program Counter)

	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
		
		
cmd_error:
	li s0,vm_ferror		;set error flag 
	jal vm_setflaga
	nop
	
	j vm_pcinc			;skip unknown byte
	nop
	
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;command processors - push/pull has special rules
    ;IX = R7	IY = R8

cmd_PHF:
	li s0,8			;ZeroPage 8 = Flags reg
	jal cmd_Stack_GetRegA
	nop
	
	andi t0,s2,0b00000001
	
	bne t0,zero,cmd_PULtoIX	;PULL
	nop
	j cmd_PSHfromIX 		;PUSH
	nop

cmd_PSH:
	jal cmd_Stack_GetReg	;T7=stack pointer S7 points to reg S2
	nop
	
cmd_PSHfromIX:
	addi t7,t7,-2			;Stack pointer -2
	
	loadLE s7,s5			;Load DS from S7/IX in LE order
	
	la a0,vm_rambaseAddr
	lw a0,(a0)				;Get Base of VM Ram
	
	storeLEa0 t7,vm_rsp		;Store Stack pointer
	
cmd_STO16:
VM_LD_IY_DE:		;Save 8 bit resuts, set flags and INC PC
	.ifdef vm_usevmem
			addi sp,sp,-4		;Subtract 4 from Stack pointer
			sw s3,0(sp)			;Put the register onto the stack
				jal vm_virmemgetphysicaliy
				nop
				StoreLE s5,s3
			lw s3,0(sp)			;Pop the register off the stack
			addi sp,sp,4		;Add 4 to the Stack pointer
			
		lw ra,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		jr ra				;Return
		nop
	.else	
		StoreLE s5,t7		;Store 16 bit result to T7/IY
		
		lw ra,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		jr ra				;Return
		nop
	.endif
	
cmd_STO:
VM_LD_IY_E:	;Save 8 bit resuts, set flags and INC PC
	.ifdef vm_usevmem
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack
			jal vm_virmemgetphysicaliy
			nop
			sb s5,(s3)
		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
	.else
		sb s5,(t7) 			;store 8 bit result in VM memory
	.endif
	
	lw ra,0(sp)		;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
cmd_PUL:
	bal cmd_Stack_GetReg	;T7=stack pointer S7 points to reg S2
	nop
	
cmd_PULtoIX:
	.ifdef vm_usevmem
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack

			jal vm_virmemgetphysicaliy
			nop
			lbu s5,(s3)
			addi s3,s3,1
			lbu s2,(s3)
			
			jal vm_virmemgetphysicalix
			nop
			sb s5,(s3)
			addi s3,s3,1
			
			sb s2,(s3)
		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
	.else
		LoadLE t7,s5		;Get S5 from stack
		StoreLE s5,s7		;Store into VM reg
	.endif
	
	addi t7,t7,2			;inc SP 
	
	la a0,vm_rambaseAddr
	lw a0,(a0)				;Get Base of VM Ram
	
	storeLEa0 t7,vm_rsp		;Store new SP
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
cmd_Stack_GetReg:		;Get S2
	move s0,s2 
	andi s0,s0,0b00000011
	sll s0,s0,1			;2 bytes per 16 bit VM register

	
cmd_Stack_GetRegA:		;Get S0
	la s7,vm_rambaseAddr
	lw s7,(s7)			;Get Base of VM Ram
	nop
	move t7,s7
	move a0,s7
	
	add s7,s7,s0		;S7 points to address of virtual reg S0
	
	LoadLEA0 vm_rsp,t7	;T7 points to stack top
	
	jr ra				;Return
	nop
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;command processors - general commands

cmd_LEA:
	move s5,t7			;Return effective address
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
cmd_MOV:
	move s5,s4
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
cmd_INC16:
	addi s5,s5,1		;INC 16 bit
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_DEC16:
	addi s5,s5,-1		;DEC 16 bit
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
cmd_ADD16:
	la t0,Mask16bit
	lw t0,(t0)
	nop
	and s5,s5,t0		;Limit our parameters to 16 bit
	and s4,s4,t0
	
	add s5,s5,s4
	
	jal CarryFromBit16plus	;Calculate the carry
	nop
	jal ZeroFlagFromHalf	;Set the Zero flag
	nop

	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
cmd_SWP16:
	jal VM_LD_IY_DE
	nop

cmd_MOV16:
	move s5,s4
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
cmd_CLR:				;Set result to 0
	li s5,0
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
cmd_SBC:				;Subtract wil carry
	jal GetCarryV1		;Get the carry
		
	andi s5,s5,255		;Limit our parameters to 8 bit
	andi s4,s4,255
	
	sub s5,s5,s4		;Subtract parameter
	sub s5,s5,v1		;Subtract Carry flag
	
	jal CarryFromBit8plus	;Set carry flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
cmd_SUB16:
	la t0,Mask16bit
	lw t0,(t0)
	nop
	and s5,s5,t0		;Limit parameters to 16 bits
	and s4,s4,t0

	sub s5,s5,s4		;Perform 16 bit Subtract 
	
	jal CarryFromBit16plus	;Set carry flag
	nop
	jal ZeroFlagFromHalf	;Set zero flag
	nop
		
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
cmd_ADD:
	andi s5,s5,255		;Limit parameters ot 8 bits
	andi s4,s4,255		;Limit parameters ot 8 bits

	add s5,s5,s4
	
	jal CarryFromBit8plus	;Set Carry flag
	nop
	jal ZeroFlagFromByte	;Set Zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
		
cmd_ADC:		;Add with carry
	jal GetCarryV1		;Get carry flag

	andi s5,s5,255		;Limit parameters to 8 bits
	andi s4,s4,255		;Limit parameters to 8 bits
	
	add s5,s5,s4		;Add parameter
	add s5,s5,v1		;Add Carry

	jal CarryFromBit8plus	;Set Carry
	nop
	jal ZeroFlagFromByte	;Set zero flag
	nop

	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
cmd_CMP16:
	la t0,Mask16bit
	lw t0,(t0)
	nop
	and s5,s5,t0		;Limit parameters to 16 bits
	and s4,s4,t0		;Limit parameters to 16 bits
	
	sub s5,s5,s4		;Perform 16 bit subtraction (Compare)
	
	jal CarryFromBit16plus	;Set Carry
	nop
	jal ZeroFlagFromHalf	;Set Zero flag
	nop
		
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	

SignExtend8:	;Sign extend S5
	andi t0,s5,0b10000000	;Test bit 7
	beq t0,zero,SignExtend8_0
	nop
	
	la t0,Mask8bitInv	
	lw t0,(t0)
	nop
	or s5,s5,t0			;Set 0xFFFFFF00
	
SignExtend8_0:
	jr ra				;Return
	nop
	

	
cmd_CMP: 	;8 bit subtract / CMP
cmd_SUB:	
	andi s5,s5,255		;Limit parameters to 8 bits
	andi s4,s4,255		;Limit parameters to 8 bits
		
	sub s5,s5,s4
	
	jal CarryFromBit8plus	;Set carry
	nop
	jal ZeroFlagFromByte	;Set zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
cmd_NEG:;
	neg s5,s5

	jal ZeroFlagFromByte	;Set zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_ROL: ;(F:CZ)
	jal GetCarryV1		;Get flag
	
	sll s5,s5,1			;Shift left 
	
	add s5,s5,v1		;Carry to bottom bit
	
	jal CarryFromBit8	;Set carry from pushed out bit
	nop

	jal ZeroFlagFromByte ;Set zero flag
	nop

	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
cmd_ROR: ;(F:CZ)
	jal GetCarryV1		;Get the carry flag

	andi s5,s5,255		;Mask to 8 bits
	
	jal CarryToBit8		;Put the carry in bit 8
	nop
	
	andi v1,s5,0b00000001	;LSB to carry
	srl s5,s5,1				;Shift 1 bit right
	
	jal ZeroFlagFromByte	;Set the Zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_ASL: ;(F:CZ)
	sll s5,s5,1			;Shift all bits left
	
	jal CarryFromBit8	;Bit 8 to carry
	nop
	
	jal ZeroFlagFromByte	;Set the zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_ASR: ;(F:CZ)
	andi s5,s5,255		;Limit parameter to 8 bits
	
	jal SignExtend8		;Sign extend it 
	nop
	
	andi v1,s5,0b00000001	;LSB to carry
	sra s5,s5,1				;Shift right
	
	jal ZeroFlagFromByte	;Set the zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
cmd_LSR: ;(F:CZ)
	andi s5,s5,255		;Limit parameter to 8 bits
	
	andi v1,s5,0b00000001	;LSB to carry
	srl s5,s5,1				;Shift right

	jal ZeroFlagFromByte	;Set the zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_INC: ;(F:Z)
	andi s5,s5,255		;Limit parameter to 8 bits
	addi s5,s5,1		;Add 1
	
	jal CarryFromBit8plus	;Set the carry
	nop
	jal ZeroFlagFromByte	;Set the zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_DEC: ;(F:Z)
	andi s5,s5,255		;Limit parameter to 8 bits
	addi s5,s5,-1		;Subtract 1

	jal CarryFromBit8plus	;Set the carry 
	nop
	jal ZeroFlagFromByte	;Set the zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_TST:
cmd_AND: ;(F:Z)
	andi s5,s5,255		;Limit parameters to 8 bits	
	andi s4,s4,255		;Limit parameters to 8 bits	
	and s5,s5,s4
	
	li v1,0				;Clear carry
	jal ZeroFlagFromByte	;Set zero flag
	nop

	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_TOR:
cmd_ORR: ;(F:Z)
	andi s5,s5,255		;Limit parameters to 8 bits	
	andi s4,s4,255		;Limit parameters to 8 bits	
	
	or s5,s5,s4
	
	li v1,0				;Clear Carry
	jal ZeroFlagFromByte	;Set zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
cmd_XOR: ;(F:Z)
	andi s5,s5,255		;Limit parameters to 8 bits	
	andi s4,s4,255		;Limit parameters to 8 bits	
	
	xor s5,s5,s4
	
	li v1,0				;Clear carry
	jal ZeroFlagFromByte	;Set zero flag
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
; Finishing Routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	

	
VM_PCINC_LD_IX_DE:	;Save 16 bit results,  and INC PC
	.ifdef vm_usevmem
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack

			jal vm_virmemgetphysicalix
			nop
			StoreLE s5,s3

		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		j vm_pcinc
		nop
	.else
		StoreLE s5,s7		;Store 16 bit result
		j vm_pcinc
		nop
	.endif
	
VM_PCINC_LD_IX_E:		;Save 8 bit results, and INC PC
	.ifdef vm_usevmem
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack
			jal vm_virmemgetphysicalix
			nop
			sb s5,(s3)
		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
	.else
		
		sb s5,(s7)
	.endif
	j vm_pcinc
	nop

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	


VM_PCINC_FZ_LD_IX_DE:	;Save 16 bit results, set flags and INC PC
	;bl GetCCR			;D7=Flags
	.ifdef vm_usevmem
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s7,0(sp)			;Put the register onto the stack
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s0,0(sp)			;Put the register onto the stack
			jal vm_virmemgetphysicalix
			nop
			StoreLE s5,s3

		lw s0,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		lw s7,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		j vm_pcinc_fzb
		nop
	.else
		StoreLE s5,s7
		j vm_pcinc_fzb
		nop
	.endif
	
	
VM_PCINC_FZ_LD_IX_E:	;Save 8 bit results, set flags and INC PC

	.ifdef vm_usevmem
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s7,0(sp)			;Put the register onto the stack
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s0,0(sp)			;Put the register onto the stack
		
			jal vm_virmemgetphysicalix
			nop
			sb s5,(s3)
			
		lw s0,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		lw s7,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
	.else
		sb s5,(s7)
	.endif
	j vm_pcinc_fzb
	nop
	
	
VM_PCINC_FZ:			;Set flags and INC PC

vm_pcinc_fzb:
	la t0,vm_rambaseAddr
	lw a0,(t0)			;Get Base of VM Ram
	nop
	lb t1,vm_rf(a0)				;Get current V-Flags
	nop
	andi t1,t1,0b11111110		;clear v-Carry
		
	beq v1,zero,vm_pcinc_ff2
	nop
	ori t1,t1,0b00000001		;Set v-Carry
	
VM_PCINC_FF2:
	andi t1,t1,0b11111101		;clear v-Zero

	beq v0,zero,vm_pcinc_ff3
	nop
	ori t1,t1,0b00000010		;Set v-Zero
VM_PCINC_FF3:
	sb t1,vm_rf(a0)
	
	
VM_PCINC:
	addi s3,s3,1			;PC++
		
VM_PC_Store:
	la t0,vm_rambaseAddr
	lw a0,(t0)				;Get Base of VM Ram
	storeLEa0 s3,vm_rpc		;Store PC
vm_pc_nostore:
	
	lw ra,0(sp)		    	;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;command processors - these commands share the same 6 bit code 
;	addressing bits define which is the command

cmd_NOP:
	move s0,s2
	andi s0,s0,255
	beq s0,zero,vm_pcinc		;NOP	   0
	nop
	
	addi s0,s0,-1 		;cp vm_RET 1
	beq s0,zero,cmd_ret
	nop					;cp vm_HLT 2
						
	addi s0,s0,-1
	nop
	
	bne s0,zero,VM_PCINC_FZ
	
cmd_HLT:
	li s0,vm_fhalt			;Set Halt flag.
	jal vm_setflaga
	nop	
	j vm_pc_store
	nop

cmd_RET:					;Also called RTS
	li s0,vm_rpc
	jal cmd_stack_getrega	;Pop from stack into virtual PC
	nop
	j cmd_pultoix			;get back pc from stack
	nop
	

VM_SetFlagA:			;set bits in vm f to the cpu S0
	la t0,vm_rambaseAddr
	lw a0,(t0)			;Get Base of VM Ram
	nop
	
	lb t1,vm_rf(a0)
	nop
	
	or t1,t1,s0			;OR virtual flags with S0
	sb t1,vm_rf(a0)
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;command processors - branch commands have special addressing

cmd_BEQ:
	andi t0,s0,0b0000010	;Test virtual Zero flag
	
	bne t0,zero,cmd_bra_true
	nop
		
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_BNE:
	andi t0,s0,0b0000010	;Test virtual Zero flag
	
	beq t0,zero,cmd_bra_true
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_BCS:
	andi t0,s0,0b0000001	;Test virtual Carry flag
	
	bne t0,zero,cmd_bra_true
	nop
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_BCC:
	andi t0,s0,0b0000001	;Test virtual Carry flag
	beq t0,zero,cmd_bra_true
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

cmd_bra_true:
	move s3,s7			;Set virtual PC to parameter address

	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
cmd_BRA16:		;Branch to 16 bit address
	addi s3,s3,1
	add s3,s3,s4		;Add parameter to the virtual PC 
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
cmd_BRA:
	move s3,s7	
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
	
vm_GetVector:	;S4=trap number
	andi s4,s4,0xFF
	sll s4,s4,2			;4 bytes per address
	
	move t0,s4
	add t0,t0,s7
	
	lw s4,(t0)			;Get Vector address
	
	jr ra				;Return
	nop


VM_CauseTrapFromOutsideVM:		;S4=trap number
	la t0,vm_rambaseAddr
	lw s3,(t0)			;Get Base of VM Ram
	lw a0,(t0)			;Get Base of VM Ram
	loadLEA0 vm_rpc,s3		;Load Little endian A3 from A0
							;(Program Counter)
	addi s3,s3,-1
			
cmd_TRP:				;D4=Trap Number
	addi s3,s3,1
	la t0,vm_trap_TableAddr
	lw s7,(t0)
	andi s4,s4,0xFF
	sll s4,s4,1			;2 bytes per trap number
	
	add s7,s7,s4

	la t0,vm_rambaseAddr
	lw s4,(t0)			;Get Base of VM Ram
	
	lbu t0,1(s7)		;Get address-H
	nop
	sll t0,t0,8
	
	lbu t1,0(s7)		;Get address-H
	nop
	
	add t1,t1,t0		;Combine address-HL
	add s4,s4,t1
	
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw s4,0(sp)			;Put the register onto the stack
	
	j cmd_call			;Call Trap address (inside the VM)
	nop

cmd_SYS:		;S4=Syscall Number
	jal vm_pcincBL
	nop
	
	la s7,vm_syscalls
	
	jal vm_getvector	;Vector S4, 4 bytes per address
	nop
		
	jr s4				;Call the system call (MIPS Code)
	nop
	
	
VM_PC_StoreBL:	
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack
		j VM_PC_Store
		nop
	
vm_pcincBL:
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack

		j vm_pcinc
		nop

cmd_PSHfromIXBL:
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack
	
		j cmd_PSHfromIX
		nop
		
cmd_BSR16:
	addi s3,s3,1
	move s7,s3
	add s7,s7,s4	;Add 16 bit offset to virtual PC
	
	
cmd_BSR:
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw s7,0(sp)			;Put the register onto the stack
		
cmd_Call:
		jal VM_PC_StoreBL	;Store updated PC in VM
		nop
		li s0,vm_rpc
		jal cmd_stack_getrega	;Prepare to push vPC onto stack
		nop
		jal cmd_PSHfromIXBL		;Store the vPC on the VM stack
		nop
		
	lw s3,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Preparation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

VM_GetAddressPrepZPDest_GetFlagC:
	jal VM_GetAddressPrepZPDestBL ;Get a zeropage destination
	nop
	j VM_GetFlagC				;Get the flag C
	nop
	
VM_GetAddress_VM_GetFlagC:
	jal vm_getaddressprepBL		;Get the address
	nop
	
VM_GetFlagC:			;Set the CPU V1 register to virtual carry
	la t0,vm_rambaseAddr
	lw a0,(t0)			;Get Base of VM Ram
	nop
	lbu v1,vm_rf(a0)
	nop
	andi v1,v1,0b00000001
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
VM_GetAddressPrep16:
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw s2,0(sp)			;Put the register onto the stack
	
		jal vm_getaddressprepBL
		nop
	lw s0,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer	
	
	andi s0,s0,0b00000011
	addi s0,s0,-1			;cp %00000001 (imm8 mode)
	bne s0,zero,lbl43438
	nop
	addi s3,s3,1			;extra byte for imm16
	
lbl43438:
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop


VM_GetAddressPrepZPDestBL:	
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack	
	
VM_GetAddressPrepZPDest: ;Patched for Zeropage dest (INC ZP / INC16)
	
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw s2,0(sp)			;Put the register onto the stack	
		jal vm_getaddressprepBL
		nop
	lw s0,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer	
	
	andi s0,s0,0b00000011
	li t0,0b00000010	;For CMP
	bne s0,t0,lbl31757
	nop
	move s7,t7			;Set Dest to Param
	move s5,s4
	
lbl31757:
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
	
VM_GetAddressPrepBL:	
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack

VM_GetAddressPrep:
	la t0,vm_rambaseAddr
	lw a0,(t0)			;Get Base of VM Ram
	nop
	move s7,a0			;Dest		S7
	move t7,a0			;Param2		T7
	addi t7,t7,2
	
	move s0,s2
	andi s0,s0,0b00000011	;=0  Accumulator Addressing
	
	beq s0,zero,lbl39573	;<>0 Other Addressing 	
	nop
	jal vm_getaddress
	nop
lbl39573:

	.ifdef vm_usevmem
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack
			jal vm_virmemgetphysicaliy
			nop
			LoadLE s3,s4	;D4=param (A6) / D5=dest (A5) / D2=command byte
			jal vm_virmemgetphysicalix
			nop
			LoadLE s3,s5 	;D4=param (A6) / D5=dest (A5)
			
			move s0,s2
			andi s0,s0,0b11111100
		
		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
	.else
		LoadLE t7,s4		;S4=param  / S5=dest / S2=command byte
		move s0,s2
		andi s0,s0,0b11111100
		
		LoadLE s7,s5 		;S4=param  / S5=dest / S0=command byte
	.endif
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Addressing mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

VM_GetAddress_ByteAddr:	;Zeropage source
	.ifdef vm_usevmem
		jal vm_virmemgethlbytea
		nop
	.else
		lbu s0,(s3)		;get a command from the program
	.endif
	
	la t0,Mask8bitInv
	lw t0,(t0)
	nop
	and t7,t7,t0		;Keep 0xFFFFF00 part
	add t7,t7,s0		;Replace bottom byte (Zero page address)
		
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
VM_GetAddress_IMM:		;8 bit immediate source (after HL / PC)
	move t7,s3			;virtual PC
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
VM_GetAddress:			;A5=Dest A6=Param
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack

	addi s3,s3,1		;vPC ++
	
	addi s0,s0,-1
	beq s0,zero,vm_getaddress_imm
	nop
	
	addi s0,s0,-1
	beq s0,zero,vm_getaddress_byteaddr
	nop
	
;VM_GetAddress_ByteParam:		;Extended addressing modes
	
	.ifdef vm_usevmem
		jal vm_virmemgethlbytea	
		nop
	.else
		lbu s0,(s3)		;Get Extended byte
		nop
	.endif
	move s5,s0			;Process DEST addressing
	move s4,s7
	jal VM_GetAddress_ByteParam2_Dest
	nop
	move s7,s4
	
	move s0,s5			;PPPP----
	srl s0,s0,4			;Process PARAM addressing
	move s4,t7			;----PPPP
	jal VM_GetAddress_ByteParam2
	nop
	move t7,s4
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	

;imm8 imm16 dest don't make sense for destination, here are some alternatives

VM_GetAddress_ByteParam2_Dest:

	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack

	la t0,Mask8bitInv	;0xFFFFFF00
	lw t1,(t0)
	
	andi s0,s0,0b00001111
	li t0,vm_prf		;Flag Destination?
	
	bne t0,s0,notvm_getaddress_rf
	nop
	
		and s4,s4,t1
		addi s4,s4,vm_rf
	
		lw ra,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		jr ra				;Return
		nop
		
notvm_getaddress_rf:
	li t0,vm_pqs			;Stack Top Addressing?
	
	bne s0,t0,notVM_GetAddress_ByteParam_Indir16_RSP
	nop
		la t0,vm_rambaseAddr
		lw a0,(t0)			;Get Base of VM Ram
		
		LoadLEA0 vm_rsp,s4	;Get Stack Address
		
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
notVM_GetAddress_ByteParam_Indir16_RSP:	
		j VM_GetAddress_ByteParam2b
		nop
		
		
VM_GetAddress_ByteParam2:
	addi sp,sp,-4		;Subtract 4 from Stack pointer
	sw ra,0(sp)			;Put the register onto the stack

	
VM_GetAddress_ByteParam2b:
	andi s0,s0,0b00001111

	la t0,Mask8bitInv	;0xFFFFFF00
	lw t0,(t0)
	nop
	and s4,s4,t0		;Keep all but bottom byte 
	add s4,s4,s0		;  (Will be replaced)
	
	li t0,vm_pr6		;0-4=direct register
		
	blt s0,t0,lbl65454	;ret c
	nop
	addi s0,s0,-5
	
		
	la t0,vm_ParameterAddressingModes
	sll s0,s0,2			;4 bytes per addressing mode
	add t0,t0,s0
	
	lw s0,(t0)			;Get the addressing sub
	nop
	jr s0				;Call it
	nop
	
lbl65454:	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
	
	
VM_GetAddress_R6:
	la t0,Mask8bitInv	;0xFFFFFF00
	lw t0,(t0)
	nop
	and s4,s4,t0
	addi s4,s4,6		;Reg R6/r7 is source
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
vm_getaddress_byteparam_imm8:
	addi s3,s3,1		;Move vPC to next byte
	move s4,s3			;This byte is parameter address
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
VM_GetAddress_ByteParam_Imm16:
	addi s3,s3,1		;Move vPC to next byte
	move s4,s3			;This word is parameter address
	addi s3,s3,1

	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
	
vm_getaddress_byteparam_immaddr8: ;Zero page addressing
	addi s3,s3,1
	
	.ifdef vm_usevmem
		jal vm_virmemgethlbytea
		nop

		la t0,Mask8bitInv	;0xFFFFFF00
		lw t0,(t0)
		nop
		and s4,s4,t0		;Keep top 24 bits
		add s4,s4,s0		;Set bottom 8 bits
	.else
		lbu t8,(s3)			;get 8 bit address
		la t0,Mask8bitInv	;0xFFFFFF00
		lw t0,(t0)
		nop
		and s4,s4,t0		;Keep top 24 bits
		add s4,s4,t8		;Set bottom 8 bits
	.endif

	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop


VM_GetAddress_ByteParam_ImmAddr16:
	.ifdef vm_usevmem
		addi s3,s3,1
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack
			jal vm_virmemgetphysical
			nop
			LoadLE s3,s4
		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		
		addi s3,s3,1
	.else
		addi s3,s3,1
		LoadLE s3,s4		;Get 16 bit address 
		
		addi s3,s3,1
	.endif
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
	
VM_GetAddress_ByteParam_Indir16_R4: ;Indirect register pair addressing 
	la a0,vm_rambaseAddr
	lw a0,(a0)			;Get Base of VM Ram
		
	LoadLEA0 vm_rr4,s4	;R4/R5
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

VM_GetAddress_ByteParam_Indir16_R6:
	la a0,vm_rambaseAddr
	lw a0,(a0)			;Get Base of VM Ram
	
	LoadLEA0 vm_rr6,s4	;R6/R7
		
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
	
VM_GetAddress_ByteParam_Indir16_R2_PostInc: ;@Rn+ postinc
	
	la a0,vm_rambaseAddr
	lw a0,(a0)			;Get Base of VM Ram
	
	LoadLEA0 vm_rr2,s4	;Get vR2
	move s0,s4
	addi s0,s0,1		;increment it
	
	StoreLEA0 s0,vm_rr2	;Store it back
		
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

VM_GetAddress_ByteParam_Indir16_R6_PostInc:
	la a0,vm_rambaseAddr
	lw a0,(a0)			;Get Base of VM Ram
	
	LoadLEA0 vm_rr6,s4	;get vR6
	
	move s0,s4
	addi s0,s0,1		;Increment it
	StoreLEA0 s0,vm_rr6	;Store it back
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
	
VM_GetAddress_ByteParam_Indir16_R2_BaseImm: ;(IX+n) type addressing
	addi s3,s3,1

	jal getsignextendedimm8		;S4=signed value from byte
	nop
		
	addi s3,s3,-1
	
	la a0,vm_rambaseAddr
	lw a0,(a0)			;Get Base of VM Ram
	nop
	move s0,a0
	
	LoadLEA0 vm_rr2,s0	;Get R2
	add s4,s4,s0		;Add offset
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop
	
	
VM_GetAddress_ByteParam_Indir16_R4_BaseImm:
	addi s3,s3,1
	
	jal getsignextendedimm8	;S4=signed value from byte
	nop
	addi s3,s3,-1
	
	la a0,vm_rambaseAddr
	lw a0,(a0)			;Get Base of VM Ram
	nop
	move s0,a0
	
	LoadLEA0 vm_rr4,s0	;Get R4	
	add s4,s4,s0		;Add the offset
	
	lw ra,0(sp)			;Pop the register off the stack
	addi sp,sp,4		;Add 4 to the Stack pointer
	jr ra				;Return
	nop

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Branch addressing
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
VM_GetAddress_Branch:
	addi s3,s3,1
	move s0,s2
	andi s0,s0,0b00000011	;Bottom 2 bits denote branch type
	beq s0,zero,vm_getaddress_branch2
	nop
	
	addi s0,s0,-1
	beq s0,zero,vm_getaddress_branch8imm
	nop
	
	addi s0,s0,-1
	beq s0,zero,vm_getaddress_branch16imm
	nop
	
;VM_GetAddress_BranchR6:	;Branch to address in R6
	
	la a0,vm_rambaseAddr
	lw a0,(a0)			;Get Base of VM Ram
	
	loadLEA0 vm_rr6,s7	;Get R6->S7 (Branch Dest)
	j vm_getaddress_branchgetf
	nop
	
VM_GetAddress_Branch16imm:
	.ifdef vm_usevmem
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw s3,0(sp)			;Put the register onto the stack
			jal vm_virmemgetphysical
			nop
			loadLE s3,s7
		lw s3,0(sp)			;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
	
		addi s3,s3,2
	.else
		loadLE s3,s7		;Get 2 bytes from program
		addi s3,s3,2
	.endif
	j vm_getaddress_branchgetf ;Get Flags
	nop
	
VM_GetAddress_Branch8imm:
	jal getsignextendedimm8	 	;D4= -128 to +127
	nop
	move s7,s3
	add s7,s7,s4		;Add offset to program counter (Branch dest)
	
vm_getaddress_branchgetf:		;Jump to S7/IX
	
	la a0,vm_rambaseAddr
	lw a0,(a0)				;Get Base of VM Ram
	nop
	lbu s0,vm_rf(a0)		;Get Flags in S0
	
	lw ra,0(sp)				;Pop the register off the stack
	addi sp,sp,4			;Add 4 to the Stack pointer
	jr ra					;Return
	nop

VM_GetAddress_Branch2:	;Skip 2 bytes
	move s7,s3
	addi s7,s7,2		;Branch Dest= vPC+2
	
	j vm_getaddress_branchgetf
	nop
	
GetSignExtendedImm8:	;D4=signed result
	.ifdef vm_usevmem
	
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw ra,0(sp)			;Put the register onto the stack

			jal vm_virmemgethlbytea
			nop
			
		lw ra,0(sp)		;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
	
		move s4,s0
	.else		
		lbu s4,(s3)		;get a byte from the program
	.endif
	addi s3,s3,1
	andi t0,s4,0b10000000 ;Test bit 7
	beq t0,zero,lbl49267
	nop
	la t0,Mask8bitInv	;0xFFFFFF00
	lw t0,(t0)
	nop
	or s4,s4,t0			;Sign extend
lbl49267:
	jr ra				;Return
	nop
	
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 	Virtual Memory addressing (optional)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
	
	.ifdef vm_usevmem
vm_VirMemGetHLByteA:
		addi sp,sp,-4		;Subtract 4 from Stack pointer
		sw ra,0(sp)			;Put the register onto the stack

			addi sp,sp,-4		;Subtract 4 from Stack pointer
			sw s3,0(sp)			;Put the register onto the stack
		
				jal vm_virmemgetphysical
				nop
				lbu s0,(s3)	
			
			lw s3,0(sp)		;Pop the register off the stack
			addi sp,sp,4		;Add 4 to the Stack pointer
		

		lw ra,0(sp)		;Pop the register off the stack
		addi sp,sp,4		;Add 4 to the Stack pointer
		jr ra				;Return
		nop
			
vm_VirMemGetPhysicalIY:		;IY=A6
		move s3,t7
		
		j vm_virmemgetphysical
		nop


vm_VirMemGetPhysicalIX:		;IX=A5
		move s3,s7
		
vm_virmemgetphysical:
		move s0,s3
		li t0,0xC000		;Keep top 2 bits (banknum)
		and s0,s0,t0
		
		sll t0,t0,12		;Switch to bank offset
		
		la a0,vm_rambaseAddr
		lw a0,(a0)			;Get Base of VM Ram
		nop
		
		addi a0,a0,vm_rbank0
		add a0,a0,s0		
		
		lbu t0,3(a0)		;Get 32 bit address from VM RAM
		nop					; Little endian
		sll t0,t0,24
		move t1,t0
		lbu t0,2(a0)
		nop
		sll t0,t0,16
		add t1,t1,t0
		lbu t0,1(a0)
		nop
		sll t0,t0,8
		add t1,t1,t0
		lbu t0,0(a0)
		nop
		add t1,t1,t0
		
		move s0,s3
		li t0,0x3FFF
		and s0,s0,t0		;Add bottom 14 bits
		add s0,s0,t1
				
		move s3,s0
	jr ra				;Return
	nop	
	
	.endif
	
	
GetCarryV1:
	la t0,vm_rambaseAddr
	lw t0,(t0)			;Get Base of VM Ram
	nop
	lb v1,vm_rf(t0)		;Get current V-Flags
	nop
	andi v1,v1,0b00000001	;Set V1 to carry
	
	jr ra				;Return
	nop
	
CarryFromBit8:
	li v1,0				;Carry Clear
	li t0,0x100			;Test bit 8
	and t0,t0,s5
	beq t0,zero,CarryFromBit8_CC	
	nop
	li v1,1				;Carry Set
CarryFromBit8_CC:	
	jr ra				;Return
	nop
	

CarryFromBit8plus:	
	li v1,0				;Carry Clear
	li t0,0xFF00		;Test top byte (for 8 bit ops)
	and t0,t0,s5
	beq t0,zero,CarryFromBit8plus_CC	
	nop
	li v1,1				;Carry Set
CarryFromBit8plus_CC:	
	jr ra				;Return
	nop
		
	
ZeroFlagFromByte:
	li v0,0				;Zero clear
	andi t0,s5,0x00FF	;Test bottom byte (for 8 bit ops)
	bne t0,zero,CarryFromByte_NZ
	nop
	li v0,1				;Zero Set
CarryFromByte_NZ:
	jr ra				;Return
	nop
		
	
CarryFromBit16:
	li v1,0				;Carry Clear
	li t0,0
	lui t0,0x0001		;Test bit 16 (for 16 bit ops)
	and t0,t0,s5
	beq t0,zero,CarryFromBit16_CC	
	nop
	li v1,1				;Carry Set
CarryFromBit16_CC:	
	jr ra				;Return
	nop
	

CarryFromBit16plus:
	li v1,0				;Carry Clear
	li t0,0
	lui t0,0xFFFF		;Test top 16 bits (for 16 bit ops)
	
	and t0,t0,s5
	beq t0,zero,CarryFromBit16plus_CC	
	nop
	li v1,1				;Carry Set
CarryFromBit16plus_CC:	
	jr ra				;Return
	nop
		
	
ZeroFlagFromHalf:
	li v0,0				;Zero clear
	andi t0,s5,0xFFFF	;Test bottom 16 bits (for 16 bit ops)
	bne t0,zero,CarryFromBitHalf_NZ
	nop
	li v0,1				;Zero Set
CarryFromBitHalf_NZ:
	jr ra				;Return
	nop
	
	
CarryToBit8:
	beq v1,zero,CarryToBit8_CC
	nop
	
	li t0,0x0100		;Set bit 8 of S5 from 'carry' in V1
	add s5,s5,t0
		
CarryToBit8_CC:
	jr ra				;Return
	nop
	

Mask16bit: .dw 0x0000FFFF
Mask16bitInv: .dw 0xFFFF0000

Mask8bit: .dw 0x000000FF
Mask8bitInv: .dw 0xFFFFFF00